import datetime
import os

from django.db import models
from django.contrib.auth.models import User
from django.conf import settings
from . import settings as lb_settings
from django.utils.encoding import python_2_unicode_compatible

from lbutils import format_filesize


try:
    AUTH_USER_MODEL = settings.AUTH_USER_MODEL
except:
    AUTH_USER_MODEL = User


def is_img(suffix):
    suffix = suffix.lower()
    return lb_settings.LBATTACHMENT_IMG_SUFFIX_LIST.count(suffix) > 0


def upload_attachment_file_path(instance, filename):
    instance.filename = os.path.basename(filename)
    path = "%s/%s/%s" % (
        instance.created_by.pk,
        datetime.date.today().strftime('%Y/%m/%d'),
        instance.filename)
    instance.suffix = os.path.splitext(filename)[1]
    instance.is_img = is_img(instance.suffix)
    return os.path.join(lb_settings.LBATTACHMENT_STORAGE_DIR, path)


@python_2_unicode_compatible
class LBAttachment(models.Model):
    attach_file = models.FileField(max_length=255, upload_to=upload_attachment_file_path)
    filename = models.CharField(max_length=255)
    suffix = models.CharField(default='', max_length=8, blank=True)
    is_img = models.BooleanField(default=False)
    num_downloads = models.IntegerField(default=0)
    description = models.TextField(default='', blank=True)
    is_active = models.BooleanField(default=False)

    created_on = models.DateTimeField(auto_now_add=True)
    created_by = models.ForeignKey(AUTH_USER_MODEL)

    def __str__(self):
        return '%s|%s' % (self.created_by.username, self.filename)

    def get_formated_filesize(self):
        return format_filesize(self.attach_file.size)



# -*- coding: UTF-8 -*-
from __future__ import unicode_literals

from django.contrib import admin

from .models import LBAttachment


class LBAttachmentAdmin(admin.ModelAdmin):
    search_fields = ('user__username', 'filename', 'act_members_param', 'notice_members_param', 'share_members_param')
    list_display = ('user', 'filename', 'is_img', 'num_downloads', 'is_active', )
    list_filter = ('is_img', 'suffix')
    raw_id_fields = ('user',)

admin.site.register(LBAttachment, LBAttachmentAdmin)



# -*- coding: UTF-8 -*-
from __future__ import unicode_literals

import json
import django
from django.test import TestCase
from django.core.urlresolvers import reverse

from lbattachment import settings as lb_settings


def resp_as_json(resp):
    if django.VERSION < (1, 9, 0):
        d = json.loads(resp.content.decode('utf8'))
    else:
        d = resp.json()
    return d


class BaseCase(TestCase):

    fixtures = ['test_lbattachment.json']

    def assertRespSucc(self, resp):
        d = resp_as_json(resp)
        self.assertTrue(d['valid'])

    def assertRespFail(self, resp):
        d = resp_as_json(resp)
        self.assertFalse(d['valid'])


class LBAttachmentTest(BaseCase):

    def login(self, username):
        self.client.login(username=username, password='138')

    def test_lbattachment_delete__(self):
        p = {'pk': 1}
        resp = self.client.get(reverse('lbattachment_delete__'), p)
        self.assertRespFail(resp)
        self.login('u1')
        resp = self.client.get(reverse('lbattachment_delete__'), p)
        self.assertRespFail(resp)
        self.login('u2')
        resp = self.client.get(reverse('lbattachment_delete__'), p)
        self.assertRespSucc(resp)

    def test_change_descn__(self):
        p = {'pk': 1}
        resp = self.client.post(reverse('lbattachment_change_descn__'), p)
        self.assertRespFail(resp)
        self.login('u1')
        resp = self.client.post(reverse('lbattachment_change_descn__'), p)
        self.assertRespFail(resp)
        self.login('u2')
        resp = self.client.post(reverse('lbattachment_change_descn__'), p)
        self.assertRespSucc(resp)

    def test_upload__(self):
        p = {}
        p['attach_file'] = open(__file__, "rb")
        resp = self.client.post(reverse('lbattachment_upload__'), p)
        p['attach_file'].close()
        self.assertRespFail(resp)
        self.login('u1')
        p['attach_file'] = open(__file__, "rb")
        resp = self.client.post(reverse('lbattachment_upload__'), p)
        p['attach_file'].close()
        self.assertRespSucc(resp)

    def test_download(self):
        lb_settings.LBATTACHMENT_X_ACCEL = True
        p = {'pk': 1}
        resp = self.client.get(reverse('lbattachment_download'), p)
        self.assertEqual(resp.status_code, 302)
        self.login('u1')
        resp = self.client.get(reverse('lbattachment_download'), p)
        self.assertEqual(resp.status_code, 200)
        lb_settings.LBATTACHMENT_X_ACCEL = False
        self.login('u1')
        resp = self.client.get(reverse('lbattachment_download'), p)
        self.assertTrue('attachments/2016/01/14/cp.png' in resp['location'])



# -*- coding: UTF-8 -*-
from __future__ import unicode_literals

from django.conf import settings


LBATTACHMENT_X_ACCEL = getattr(settings, 'LBATTACHMENT_X_ACCEL', False)
LBATTACHMENT_STORAGE_DIR = getattr(settings, 'LBATTACHMENT_STORAGE_DIR', 'lbattachments')
LBATTACHMENT_STORAGE_DIR = getattr(settings, 'LBATTACHMENT_STORAGE_DIR', 'lbattachments')
LBATTACHMENT_MEDIA_URL = getattr(settings, 'LBATTACHMENT_MEDIA_URL', 'lbattachments')
LBATTACHMENT_IMG_SUFFIX_LIST = getattr(settings, 'LBATTACHMENT_IMG_SUFFIX_LIST', ['png', 'gif', 'jpg', 'jpeg'])
LBATTACHMENT_IMG_SUFFIX_LIST = [e.lower() for e in LBATTACHMENT_IMG_SUFFIX_LIST]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.translation import ugettext as _
from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth.decorators import login_required
from django.shortcuts import get_object_or_404
from django.http import HttpResponse
from django.http import JsonResponse
from django.shortcuts import redirect

from .forms import LBAttachmentForm
from .models import LBAttachment
from . import settings as lb_settings


@csrf_exempt
def upload__(request):
    ctx = {}
    if not request.user.is_authenticated():
        ctx['valid'] = False
        ctx['errors'] = _('Please login first.')
        return JsonResponse(ctx)
    form = LBAttachmentForm()
    if request.method == "POST":
        form = LBAttachmentForm(request.POST, request.FILES)
        if form.is_valid():
            obj = form.save(commit=False)
            obj.created_by = request.user
            obj.save()
            ctx['valid'] = True
            ctx['file'] = {
                'pk': obj.pk,
                'fn': obj.filename,
                'url': obj.attach_file.url,
                'descn': obj.description}
        else:
            ctx['valid'] = False
            ctx['errors'] = form.errors_as_text()
    return JsonResponse(ctx)


@login_required
def download(request):
    pk = request.GET.get('pk')
    response = HttpResponse('')
    obj = get_object_or_404(LBAttachment, pk=pk)
    fn = obj.attach_file.name
    if not lb_settings.LBATTACHMENT_X_ACCEL:
        return redirect(obj.attach_file.url)
    response['X-Accel-Redirect'] = fn.encode('UTF-8')
    response["Content-Disposition"] = "attachment; filename={0}".\
        format(obj.filename).encode('utf-8')
    response['Content-Type'] = ''
    return response


@csrf_exempt
def delete__(request):
    ctx = {}
    attachment_pk = request.POST.get('pk', 0) or request.GET.get('pk', 0)
    attachment = LBAttachment.objects.get(pk=attachment_pk)
    if (attachment.created_by != request.user):
        ctx['valid'] = False
        ctx['errors'] = _('no right')
    else:
        attachment.delete()
        ctx['valid'] = True
    return JsonResponse(ctx)


@csrf_exempt
def change_descn__(request):
    ctx = {'valid': False, 'errors': ''}
    pk = request.POST.get('pk', 0) or request.GET.get('pk', 0)
    attachment = get_object_or_404(LBAttachment, pk=pk)
    if not attachment:
        ctx['valid'] = False
        ctx['errors'] = _("This file could't be find.")
        return JsonResponse(ctx)
    if attachment.created_by != request.user:
        ctx['valid'] = False
        ctx['errors'] = _('no right')
        return JsonResponse(ctx)
    if request.method == "POST":
        attachment.description = request.POST.get('descn', '')
        ctx['valid'] = True
        attachment.save()
    return JsonResponse(ctx)



from django.conf.urls import include, url


from django.contrib import admin
admin.autodiscover()

urlpatterns = [
    url(r"^", include("lbattachment.urls")),
    url(r'^admin/', include(admin.site.urls)),
]



from django.conf.urls import url

from . import views


urlpatterns = [
    url('^upload__/$', views.upload__, name='lbattachment_upload__'),
    url('^delete__/$', views.delete__, name='lbattachment_delete__'),
    url('^change_descn__/$', views.change_descn__, name='lbattachment_change_descn__'),
    url('^download/$', views.download, name='lbattachment_download'),
]



from django import forms

from lbutils import FormHelperMixin
from .models import LBAttachment


class LBAttachmentForm(FormHelperMixin, forms.ModelForm):

    class Meta:
        model = LBAttachment
        fields = ('attach_file',)
